"""Widgets subpackage for LikX."""

from .color_picker import InlineColorPicker
from .save_handler import SaveHandler
from .tab_manager import TabContent, TabManager

__all__ = ["InlineColorPicker", "SaveHandler", "TabContent", "TabManager"]
